"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decoratorPlugin = void 0;
var unist_util_visit_1 = __importDefault(require("unist-util-visit"));
var regexp_match_indices_1 = __importDefault(require("regexp-match-indices"));
var splitTextIntoNodes = function (text, decorator) {
    decorator.match.lastIndex = 0;
    var nodes = [];
    var lastIndex = 0;
    while (true) {
        var result = regexp_match_indices_1.default(decorator.match, text);
        if (!result) {
            break;
        }
        var captureGroupIndex = decorator.captureGroupIndex || 0;
        var _a = result.indices[captureGroupIndex], captureGroupStartIndex = _a[0], captureGroupEndIndex = _a[1];
        var matchedString = text.slice(captureGroupStartIndex, captureGroupEndIndex);
        nodes.push({
            type: 'text',
            value: text.substring(lastIndex, captureGroupStartIndex),
        }, {
            type: 'element',
            tagName: decorator.component,
            properties: decorator.properties,
            children: [
                {
                    type: 'text',
                    value: matchedString,
                },
            ],
        });
        lastIndex = captureGroupEndIndex;
        if (!decorator.match.global) {
            break;
        }
    }
    if (text.length > lastIndex) {
        nodes.push({
            type: 'text',
            value: text.substring(lastIndex),
        });
    }
    decorator.match.lastIndex = 0;
    return nodes;
};
exports.decoratorPlugin = function (options) {
    var _a;
    if (!((_a = options.decorators) === null || _a === void 0 ? void 0 : _a.length)) {
        return;
    }
    var visitor = function (node) {
        var decorator = options.decorators.find(function (entry) {
            return node.type === 'text' && entry.match.test(node.value);
        });
        if (!decorator) {
            return;
        }
        node.type = 'element';
        node.tagName = 'span';
        node.children = splitTextIntoNodes(node.value, decorator);
        return unist_util_visit_1.default.SKIP;
    };
    return function (tree) {
        unist_util_visit_1.default(tree, visitor);
    };
};
